#
# PST module DrawCropMarks - draw cross marks for paper cutter
# 
# (c) 2002  Tomas Valla   <valla@atrey.karlin.mff.cuni.cz>
# 
# This software may be used and distributed under the terms
# of the GNU General Public License. See COPYING for details.
#
#
#
#
#
# Options:
# lw|line_width = <unit>
# ml|mark_len = <unit>      how long one mark should be
# es|edge_space = <unit>    how far from paper
#


package DrawCM;
use sizes;
@ISA = qw(OM);
use strict;


sub new {
	my $proto = shift;
	my $class = ref($proto) || $proto;
	my $self = OM->new;

	$self->name("DrawCM");
	$self->{LINE_WIDTH} = 1;
	$self->{MARK_LEN} = 50;
	$self->{EDGE_SPACE} = 10;

	return bless($self,$class);
}

sub parse_options {
	my $self = shift;
	my $str = "";

	$self->log(4,"parsing options");
	while (my $x = shift) { $str .= $x }
	$str =~ s/\s//g;
	foreach my $e (split /;/, $str) {
		if ($e =~ /^(lw|line_width)\=([\w\.]+)$/) {
			my $u = unit_size($2);
			$self->die($u) if $u =~ /^ERROR/;
			$self->{LINE_WIDTH} = $u;
		} elsif ($e =~ /^(ml|mark_length)\=(\w+)$/) {
			my $u = unit_size($2);
			$self->die($u) if $u =~ /^ERROR/;
			$self->{MARK_LEN} = $u;
		} elsif ($e =~ /^(es|edge_space)\=(\w+)$/) {
			my $u = unit_size($2);
			$self->die($u) if $u =~ /^ERROR/;
			$self->{EDGE_SPACE} = $u;
		} else {
			$self->die("bad option ($e)\n");
		}
	}
	$self->log(2,"line width ",$self->{LINE_WIDTH},", mark length ",$self->{MARK_LEN},
		", space at edge ",$self->{EDGE_SPACE});
}

sub page_size {
	my $self = shift;
	my $page = shift;

	return 0 if $page<0 or $page>$self->{PREV_PAGES};
	my $plus = 2*($self->{EDGE_SPACE}+$self->{MARK_LEN});  # marks use some space...
	$self->{PAGE_WIDTH}{$page} = $self->{PREV_PAGE_WIDTH}{$page} + $plus;
	$self->{PAGE_HEIGHT}{$page} = $self->{PREV_PAGE_HEIGHT}{$page} + $plus;
	return 1;
}

sub write_page_prolog {
	my $self = shift;
	my $page = shift;
	my $f = $self->{OUTPUT};
	my $es = $self->{EDGE_SPACE};
	my $ml = $self->{MARK_LEN};
	my $h = $self->{PREV_PAGE_HEIGHT}{$page};
	my $w = $self->{PREV_PAGE_WIDTH}{$page};

	$self->log(4,"writing page $page prolog (with marks drawing)");
	print $f
		"userdict/PSTdict save put\nPSTmatrix setmatrix\n",   # save graphic and set line width
		$es+$ml," ",$es+$ml," translate\n",  # make space for cropmarks
		"gsave\n",
		$self->{LINE_WIDTH}," setlinewidth\n",
		"newpath -$es 0 moveto ",-($es+$ml)," 0 lineto stroke\n",  # ll horizontal
		"newpath 0 -$es moveto 0 ",-($es+$ml)," lineto stroke\n",  # ll vertical
		"newpath ", $w+$es," 0 moveto ",$w+$es+$ml," 0 lineto stroke\n",  # lr horiz
		"newpath $w -$es moveto $w ",-($es+$ml)," lineto stroke\n",  # lr vert
		"newpath ",$w+$es," $h moveto ",$w+$es+$ml," $h lineto stroke\n",  # ur horiz
		"newpath $w ",$h+$es," moveto $w ",$h+$es+$ml," lineto stroke\n",  # ur vert
		"newpath -$es $h moveto ",-($es+$ml)," $h lineto stroke\n",  # ul horiz
		"newpath 0 ",$h+$es," moveto 0 ",$h+$es+$ml," lineto stroke\n",  # ul vert
		"grestore\n",
                "userdict/PSTmatrix matrix currentmatrix put\n",
                #"userdict/PSTclip{0 0 moveto\n",
                #$pw," 0 rlineto 0 ",$ph," rlineto -",$pw," 0 rlineto\n",
                #"closepath}put initclip\n",
                "PSTxform concat\n";

}

sub write_page_epilog {
	my $self = shift;
	my $f = $self->{OUTPUT};

	print $f
		"PSTdict restore\n";
}



1;  # no init...
