#
# PST module Rotate - rotate page by some angle
# 
# (c) 2002  Tomas Valla   <valla@atrey.karlin.mff.cuni.cz>
# 
# This software may be used and distributed under the terms
# of the GNU General Public License. See COPYING for details.
#
#
#
# options:
# a|angle = <int>


package Rotate;
use OM;
@ISA = qw(OM);
use strict;

sub new {
	my $proto = shift;
	my $class = ref($proto) || $proto;
	my $self = OM->new;

	$self->name("Rotate");
	$self->{ANGLE} = undef;

	return bless($self,$class);
}

sub parse_options {
	my $self = shift;

	$self->log(4,"parsing options");
	while (my $str = shift) {
		foreach my $e (split /;/, $str) {
			if ($e =~ /^\s*(a|angle)\s*\=\s*(-?\d+)\s*$/) {
				my $a = $2 % 360;
				if ($a<0) { $a += 360 }
				$self->{ANGLE} = $a;
			} else {
				$self->die("bad option ($e)\n");
			}
		}
	}
	$self->log(2,"rotation angle is ",$self->{ANGLE});
}

sub page_size {
	my $self = shift;
	my $page = shift;

	return 0 if $page<0 or $page>$self->{PREV_PAGES};

	my $deg_ang = $self->{ANGLE};
	my $pi = atan2(1,1)*4;
	my $ang = $deg_ang * $pi / 180;  # in radians
	my $h = $self->{PREV_PAGE_HEIGHT}{$page};
	my $w = $self->{PREV_PAGE_WIDTH}{$page};
	my $nh;
	my $nw;

	# this is ugly...
	if ($deg_ang <= 90) {
		$nw = $h*sin($ang) + $w*cos($ang);
		$nh = $w*sin($ang) + $h*cos($ang);
	} elsif ($deg_ang <= 180) {
		$nw = $w*sin($ang-$pi/2) + $h*cos($ang-$pi/2);
		$nh = $h*sin($ang-$pi/2) + $w*cos($ang-$pi/2);
	} elsif ($deg_ang <= 270) {
		$nw = $h*sin($ang-$pi) + $w*cos($ang-$pi);
		$nh = $w*sin($ang-$pi) + $h*cos($ang-$pi);
	} else {
		$nh = $w*cos($ang-$pi*3/2) + $h*sin($ang-$pi*3/2);
		$nw = $h*cos($ang-$pi*3/2) + $w*sin($ang-$pi*3/2);
	}
		
	$self->{PAGE_HEIGHT}{$page} = $nh;
	$self->{PAGE_WIDTH}{$page} = $nw;
	return 1;
}

sub write_page_prolog {
	my $self = shift;
	my $page = shift;
	my $f = $self->{OUTPUT};
	my $deg_ang = $self->{ANGLE};
	my $pi = atan2(1,1)*4;
	my $ang = $deg_ang * $pi / 180;
	my $h = $self->{PREV_PAGE_HEIGHT}{$page};
	my $w = $self->{PREV_PAGE_WIDTH}{$page};

	my $x;
	my $y;
	if ($deg_ang <=90) {
		$x = $h*sin($ang);
		$y = 0;
	} elsif ($deg_ang <=180) {
		$x = $self->{PAGE_WIDTH}{$page};
		$y = $h*sin($ang-$pi/2);
	} elsif ($deg_ang <=270) {
		$x = $w*cos($ang-$pi);
		$y = $self->{PAGE_HEIGHT}{$page};
	} else {
		$x = 0;
		$y = $w*cos($ang-$pi*3/2);
	}
	#$x = int $x;
	#$y = int $y;

	$self->log(4,"writing page $page prolog (shift and rotation)");
	# since the rot. center is at (0,0), we must shift the page to stay in visible area
	print $f "userdict/PSTdict save put\nPSTmatrix setmatrix\n",
		"$x $y translate ",  # shift the page
		$self->{ANGLE}," rotate\n",
                "userdict/PSTmatrix matrix currentmatrix put\n",
                #"userdict/PSTclip{0 0 moveto\n",
                #$pw," 0 rlineto 0 ",$ph," rlineto -",$pw," 0 rlineto\n",
                #"closepath}put initclip\n",
                "PSTxform concat\n";

}

sub write_page_epilog {
	my $self = shift;
	my $page = shift;
	my $f = $self->{OUTPUT};

	$self->log(4,"writing page $page epilog");
	print $f "PSTdict restore\n"
}



1; # no init...

